@@ -27,19 +27,33 @@ import sys
 import codecs
 sys.stdout = codecs.getwriter('utf8')(sys.stdout)
 
-__VERSION__ = '0.3.6'
+__VERSION__ = '0.3.8'
 
 DESCRIPTION = 'Bumpversion: v{} (using Python v{})'.format(
     __VERSION__,
     sys.version.split("\n")[0].split(" ")[0],
 )
 
-
-class Git(object):
+class BaseVCS(object):
 
     @classmethod
     def is_usable(cls):
-        return os.path.isdir(".git")
+        try:
+            return subprocess.call(
+                cls._TEST_USABLE_COMMAND,
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE
+            ) == 0
+        except OSError as e:
+            if e.errno == 2:
+                # mercurial is not installed then, ok.
+                return False
+            raise
+
+
+class Git(BaseVCS):
+
+    _TEST_USABLE_COMMAND = ["git", "rev-parse", "--git-dir"]
 
     @classmethod
     def assert_nondirty(cls):
@@ -58,7 +72,7 @@ class Git(object):
     def latest_tag_info(cls):
         try:
             # git-describe doesn't update the git-index, so we do that
-            subprocess.check_call(["git", "update-index", "--refresh"])
+            subprocess.check_output(["git", "update-index", "--refresh"])
 
             # get info about the latest tag in git
             describe_out = subprocess.check_output([
@@ -77,7 +91,7 @@ class Git(object):
 
         info = {}
 
-        if describe_out[-1] is "dirty":
+        if describe_out[-1].strip() == "dirty":
             info["dirty"] = True
             describe_out.pop()
 
@@ -92,22 +106,20 @@ class Git(object):
 
     @classmethod
     def add_path(cls, path):
-        subprocess.check_call(["git", "add", "--update", path])
+        subprocess.check_output(["git", "add", "--update", path])
 
     @classmethod
     def commit(cls, message):
-        subprocess.check_call(["git", "commit", "-m", message.encode('utf-8')])
+        subprocess.check_output(["git", "commit", "-m", message.encode('utf-8')])
 
     @classmethod
     def tag(cls, name):
-        subprocess.check_call(["git", "tag", name])
+        subprocess.check_output(["git", "tag", name])
 
 
-class Mercurial(object):
+class Mercurial(BaseVCS):
 
-    @classmethod
-    def is_usable(cls):
-        return os.path.isdir(".hg")
+    _TEST_USABLE_COMMAND = ["hg", "root"]
 
     @classmethod
     def latest_tag_info(cls):
@@ -132,11 +144,11 @@ class Mercurial(object):
 
     @classmethod
     def commit(cls, message):
-        subprocess.check_call(["hg", "commit", "-m", message.encode('utf-8')])
+        subprocess.check_output(["hg", "commit", "-m", message.encode('utf-8')])
 
     @classmethod
     def tag(cls, name):
-        subprocess.check_call(["hg", "tag", name])
+        subprocess.check_output(["hg", "tag", name])
 
 VCS = [Git, Mercurial]
 
@@ -224,8 +236,43 @@ class Version(object):
             elif bumped:
                 self._parsed[label].zero()
 
+OPTIONAL_ARGUMENTS_THAT_TAKE_VALUES = [
+    '--config-file',
+    '--current-version',
+    '--message',
+    '--new-version',
+    '--parse',
+    '--serialize',
+    '--tag-name',
+]
 
-def main(args=None):
+
+def split_args_in_optional_and_positional(args):
+    # manually parsing positional arguments because stupid argparse can't mix
+    # positional and optional arguments
+
+    positions = []
+    for i, arg in enumerate(args):
+
+        previous = None
+
+        if i > 0:
+            previous = args[i-1]
+
+        if ((not arg.startswith('--'))  and
+            (previous not in OPTIONAL_ARGUMENTS_THAT_TAKE_VALUES)):
+            positions.append(i)
+
+    positionals = [arg for i, arg in enumerate(args) if i in positions]
+    args = [arg for i, arg in enumerate(args) if i not in positions]
+
+    return (positionals, args)
+
+def main(original_args=None):
+
+    positionals, args = split_args_in_optional_and_positional(
+      sys.argv[1:] if original_args is None else original_args
+    )
 
     parser1 = argparse.ArgumentParser(add_help=False)
 
@@ -270,24 +317,12 @@ def main(args=None):
                          help='Version that needs to be updated', required=False)
     parser2.add_argument('--parse', metavar='REGEX',
                          help='Regex parsing the version string',
-                         default='(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)')
+                         default=defaults.get("parse", '(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)'))
     parser2.add_argument('--serialize', metavar='FORMAT',
                          help='How to format what is parsed back to a version',
-                         default='{major}.{minor}.{patch}')
+                         default=defaults.get("serialize", str('{major}.{minor}.{patch}')))
 
-    parser2_2 = argparse.ArgumentParser(
-        description=DESCRIPTION,
-        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
-        conflict_handler='resolve',
-        add_help=False,
-        parents=[parser2],
-    )
-    parser2_2.add_argument('part', help='Part of the version to be bumped.', nargs='?')
-
-    known_args, remaining_argv = parser2_2.parse_known_args(remaining_argv)
-
-    if known_args.part:
-        remaining_argv[0:0] = [known_args.part]
+    known_args, remaining_argv = parser2.parse_known_args(args)
 
     defaults.update(vars(known_args))
 
@@ -304,7 +339,10 @@ def main(args=None):
 
     if not 'new_version' in defaults and known_args.current_version:
         v.parse(known_args.current_version)
-        v.bump(known_args.part)
+
+        if len(positionals) > 0:
+            v.bump(positionals[0])
+
         defaults['new_version'] = v.serialize()
 
     parser3 = argparse.ArgumentParser(
@@ -355,13 +393,12 @@ def main(args=None):
     parser3.add_argument('part',
                          help='Part of the version to be bumped.')
     parser3.add_argument('files', metavar='file',
-                         nargs='+' if len(files) == 0 else '*',
+                         nargs='*',
                          help='Files to change', default=files)
 
-    args = parser3.parse_args(remaining_argv)
+    args = parser3.parse_args(remaining_argv + positionals)
 
-    if len(args.files) is 0:
-        warnings.warn("No files specified")
+    files = files or positionals[1:]
 
     for vcs in VCS:
         if vcs.is_usable():
@@ -369,7 +406,7 @@ def main(args=None):
             break
 
     # make sure files exist and contain version string
-    for path in args.files:
+    for path in files:
         with io.open(path, 'rb') as f:
             before = f.read().decode('utf-8')
 
@@ -377,7 +414,7 @@ def main(args=None):
             args.current_version, path)
 
     # change version string in files
-    for path in args.files:
+    for path in files:
         with io.open(path, 'rb') as f:
             before = f.read().decode('utf-8')
 
@@ -390,7 +427,7 @@ def main(args=None):
             with io.open(path, 'wt', encoding='utf-8') as f:
                 f.write(after)
 
-    commit_files = args.files
+    commit_files = files
 
     if config:
         config.remove_option('bumpversion', 'new_version')
@@ -413,6 +450,9 @@ def main(args=None):
             commit_files.append(known_args.config_file)
 
     if args.commit:
+
+        assert vcs.is_usable(), "Did find '{}' unusable, unable to commit.".format(vcs.__name__)
+
         if not args.dry_run:
             for path in commit_files:
                 vcs.add_path(path)
@@ -428,3 +468,5 @@ def main(args=None):
 
             if args.tag:
                 vcs.tag(args.tag_name.format(**vcs_context))
+
+
